==> hermes-6.0.1.52.tar.gz: hermes-6.0.1.52/lib/hermes/rex_client/client.rb <==
require "rest-client"
require "base64"
require "nokogiri"
require "addressable/uri"

module Hermes
  class RexClient::Client
    class ClientError < StandardError
      attr_reader :response

      def initialize(response)
        super(message)
        @response = response
        if false
          begin
            message = "#{error_message} - CODE '#{code}'"
          rescue => e
            message = "Error while call API, Unparsable response (error: #{e.inspect}) : #{self.response.to_s.force_encoding("UTF-8")}"
          end
        end
        message = "Error : #{response.to_s.force_encoding("UTF-8")}"

        super(message)
      end

      def request
        response.request
      end
    end

    class Forbidden < StandardError
    end

    class NotFound < StandardError
    end

    attr_accessor :auth_url
    attr_accessor :base_url
    attr_accessor :ed_entity
    attr_accessor :ed_lessor
    attr_accessor :ed_contact
    attr_accessor :base_url
    attr_accessor :client_id
    attr_accessor :client_secret
    attr_accessor :username
    attr_accessor :password
    attr_accessor :logger
    attr_accessor :token

    def initialize(attrs = {})
      @auth_url = attrs.fetch(:base_url, RexClient.config.auth_url)
      @base_url = attrs.fetch(:base_url, RexClient.config.base_url)
      @ed_entity = attrs.fetch(:ed_entity, RexClient.config.ed_entity)
      @ed_lessor = attrs.fetch(:ed_lessor, RexClient.config.ed_lessor)
      @ed_contact = attrs.fetch(:ed_lessor, RexClient.config.ed_contact)
      @client_id = attrs.fetch(:base_url, RexClient.config.client_id)
      @client_secret = attrs.fetch(:base_url, RexClient.config.client_secret)
      @username = attrs.fetch(:username, RexClient.config.username)
      @password = attrs.fetch(:password, RexClient.config.password)
      @logger = attrs.fetch(:logger, RexClient.config.logger)
      if @tagset_id = attrs.fetch(:tagset_id, nil)
        @csv_content = []
        @errors = []
        @email_mail_to = attrs.fetch(:email_mail_to,  ['support.trusted@be-ys-trusted-solutions.com'] )
      end 
    end

    def log_error(error, classname = nil, id = nil)
      return unless classname && id

      instance = classname.constantize.find(id)
      return unless instance
      instance.add_error(error)
    end

    def log_success(classname = nil, id = nil)
      return unless classname && id

      instance = classname.constantize.find(id)
      return unless instance
      instance.set(last_send_has_error: false, updated_at: DateTime.current)
    end

    def log_puts(line, error = false)
      if @logger
        @logger.info line
      end 

      if @csv_content
        @csv_content.push(line)
        @errors.push(line) if error
      end  

      puts(line)
    end

    def save_log_and_send_email
      return unless @csv_content

      csv = CSV.generate(col_sep: ";",encoding: Encoding::UTF_8) do |data|
        @csv_content.each do |row|
          data<<[row]
        end
      end

      support = User.find_by(:email=> "support@plugandwork.fr")
      current_time = DateTime.current.strftime('%Y-%m-%d-%H-%M-%S')

      csv_name = "rex_client_log_#{current_time}"
      dcsv = Doc.create(responsible: support.id, type: "file", title: "#{csv_name}.csv", tagset_ids: [@tagset_id])
      dcsv.blob = csv
      dcsv.save

      error_length = @errors&.length

      doc_mail = Doc.create(
        responsible: support.id,
        type: 'email', 
        title: "Rapport Syncro REX #{HERMES_CONFIG['environment']} : #{current_time} #{error_length} erreurs", 
      )
      doc_mail.superfields['mail_to'] = (@email_mail_to).join(',')
      doc_mail.superfields['mail_from'] = support.email

      body = "Bonjour, <br/>"
      body += "Voici le lien vers le log de la syncro REX : <a href='#{APP_CONFIG["app_host"]}/documents/#{dcsv.id}'>#{dcsv.title}</a><br/>"
      body += "Nombre d'erreurs : #{error_length} <br/>"

      if error_length > 0
        body += "Les #{[100, error_length].min} premières erreurs : <br/>"
        @errors.take(100).each do |error|
          body += "#{error}<br/>"
        end 
          
        if error_length > 100
          body += "Pour voir les erreurs suivantes, il faut aller sur l'admin PAW.<br/>"
        end 
      else
        body += "Aucune erreur.<br/>"
      end 

      doc_mail.body = body
      doc_mail.save

      # Envoi du mail
      Agent.regular_mail(doc_mail.id.to_s, 
        to: doc_mail.superfields['mail_to'], 
        from: doc_mail.superfields['mail_from'], 
        title: doc_mail.title, 
        content: doc_mail.body
      ).deliver_later
    end 

    def get_token
      @token = RestClient.post(
        @auth_url,
        headers: { "Content-Type" => "application/x-www-form-urlencoded" },
        grant_type: "password",
        client_id: @client_id,
        client_secret: @client_secret,
        username: @username,
        password: @password,
        log: logger,
      ).json_to_h[:access_token]
      puts "Got Rex token : #{@token}"
      @token
    end

    def request_header
      {
        "Content-Type" => "application/x-www-form-urlencoded",
        "Authorization" => "Bearer #{@token}",
      }
    end

    def get_structure_ed(type)
      return @ed_entity if type == "entity"
      return @ed_lessor if type == "lessor"
      return @ed_contact if type == "contact"
      return
    end

    def find_base_structure_by_id(id, type = "entity")
      get_token if !@token

      begin
        return RestClient.get(
          "#{@base_url}/entityDefinition/#{get_structure_ed(type)}/entity/#{id}",
          request_header,
        ).json_to_h
      rescue RestClient::ExceptionWithResponse => e
        return;
      end
    end

    def find_entity_by_id(id)
      find_base_structure_by_id(id, "entity")
    end

    def find_lessor_by_id(id)
      find_base_structure_by_id(id, "lessor")
    end

    def find_structure_by_id(id)
      find_base_structure_by_id(id, "entity") || find_base_structure_by_id(id, "lessor")
    end

    def get_page(opt, type = "entity", page = 1)
      if @logger
        @logger.info "Getting page : #{page}"
      end 
      size = 10
      from = (page - 1) * size
      
      get_token if !@token
      
      query = {
        from: from,
        size: size,
        sortField: "fields.id_cristal",
        sortOrder: 'ASC',
      }.merge(opt).to_query

      start = Time.now

      response = begin
        RestClient.get(
          "#{@base_url}/entityDefinition/#{get_structure_ed(type)}/entity?#{query}",
          request_header,
        )
        rescue RestClient::ExceptionWithResponse => e
          e.response
      end
      

      finish = Time.now
      diff = finish - start

      log_puts "#{get_structure_ed(type)}/entity?#{query} #{diff} sec"

      if response.code == 401
        get_token
        return get_page(opt, type, page) 
      end

      response.body.json_to_h
    end

    def search_base(opt, type = "entity", callback = nil, page = 1, previous_result = [], total = 0)
      start = Time.now
      size = 10
      from = (page - 1) * size
      
      if total && total < from
        return previous_result
      end
      
      get_token if !@token
      
      query = {
        from: from,
        size: size,
        sortField: "fields.id_cristal",
        sortOrder: 'ASC',
      }.merge(opt).to_query

      response = begin
        RestClient.get(
          "#{@base_url}/entityDefinition/#{get_structure_ed(type)}/entity?#{query}",
          request_header,
        )
        rescue RestClient::ExceptionWithResponse => e
          e.response
      end

      if response.code == 401
        get_token
        return search_base(opt, type, callback, page, previous_result, total) 
      end

      r = response.body.json_to_h

      if callback
        r[:items].each do |item|
          callback.call(item)
        end
        
        finish = Time.now
        diff = finish - start

        puts("Sync #{type} #{[from + size, r[:total]].min}/#{r[:total]} #{diff} sec")
      end

      new_result = (callback) ?  [] : previous_result + r[:items]
      
      search_base(opt, type, callback, page + 1, new_result, r[:total])
    end

    def search_entities(opt = {})
      search_base(opt, "entity")
    end

    def search_lessors(opt = {})
      search_base(opt, "lessor")
    end

    def search_contacts(opt = {})
      search_base(opt, "contact")
    end

    def search_structures(opt = {})
      search_base(opt, "entity") + search_base(opt, "lessor")
    end

    def traverse_base(opt, type, callback)
      total = get_page(opt, type, 1)[:total]
      totalPages = (total / 10).floor + 1
      
      (1..totalPages).to_a.each do |page|
        start = Time.now
        reponse = get_page(opt, type, page)
        if callback
          reponse[:items].each do |item|
            callback.call(item)
          end
          
          finish = Time.now
          diff = finish - start

          puts("Sync #{type} #{[page * 10, total].min}/#{total} #{diff} sec")
        end
      end
    end

    def traverse_structures(opt = {}, opt2 = {}, callback = nil)
      traverse_base(opt, 'lessor', callback) unless opt2[:dont_update_lessor] 
      traverse_base(opt, 'entity', callback) unless opt2[:dont_update_entity] 
    end

    def traverse_contacts(opt = {}, opt2 = {}, callback = nil)
      traverse_base(opt, "contact", callback)
    end
  end
end

==> hermes-6.0.2.29.tar.gz: hermes-6.0.2.29/lib/hermes/rex_client/client.rb <==
require "rest-client"
require "base64"
require "nokogiri"
require "addressable/uri"

module Hermes
  class RexClient::Client
    class ClientError < StandardError
      attr_reader :response

      def initialize(response)
        super(message)
        @response = response
        if false
          begin
            message = "#{error_message} - CODE '#{code}'"
          rescue => e
            message = "Error while call API, Unparsable response (error: #{e.inspect}) : #{self.response.to_s.force_encoding("UTF-8")}"
          end
        end
        message = "Error : #{response.to_s.force_encoding("UTF-8")}"

        super(message)
      end

      def request
        response.request
      end
    end

    class Forbidden < StandardError
    end

    class NotFound < StandardError
    end

    attr_accessor :auth_url
    attr_accessor :base_url
    attr_accessor :ed_entity
    attr_accessor :ed_lessor
    attr_accessor :ed_contact
    attr_accessor :base_url
    attr_accessor :client_id
    attr_accessor :client_secret
    attr_accessor :username
    attr_accessor :password
    attr_accessor :logger
    attr_accessor :token

    def initialize(attrs = {})
      @auth_url = attrs.fetch(:base_url, RexClient.config.auth_url)
      @base_url = attrs.fetch(:base_url, RexClient.config.base_url)
      @ed_entity = attrs.fetch(:ed_entity, RexClient.config.ed_entity)
      @ed_lessor = attrs.fetch(:ed_lessor, RexClient.config.ed_lessor)
      @ed_contact = attrs.fetch(:ed_lessor, RexClient.config.ed_contact)
      @client_id = attrs.fetch(:base_url, RexClient.config.client_id)
      @client_secret = attrs.fetch(:base_url, RexClient.config.client_secret)
      @username = attrs.fetch(:username, RexClient.config.username)
      @password = attrs.fetch(:password, RexClient.config.password)
      @logger = attrs.fetch(:logger, RexClient.config.logger)
      if @tagset_id = attrs.fetch(:tagset_id, nil)
        @csv_content = []
        @errors = []
        @email_mail_to = attrs.fetch(:email_mail_to,  ['support.trusted@be-ys-trusted-solutions.com'] )
      end 
    end

    def log_error(error, classname = nil, id = nil)
      return unless classname && id

      instance = classname.constantize.find(id)
      return unless instance
      instance.add_error(error)
    end

    def log_success(classname = nil, id = nil)
      return unless classname && id

      instance = classname.constantize.find(id)
      return unless instance
      instance.set(last_send_has_error: false, updated_at: DateTime.current)
    end

    def log_puts(line, error = false)
      if @logger
        @logger.info line
      end 

      if @csv_content
        @csv_content.push(line)
        @errors.push(line) if error
      end  

      puts(line)
    end

    def save_log_and_send_email
      return unless @csv_content

      csv = CSV.generate(col_sep: ";",encoding: Encoding::UTF_8) do |data|
        @csv_content.each do |row|
          data<<[row]
        end
      end

      support = User.find_by(:email=> "support@plugandwork.fr")
      current_time = DateTime.current.strftime('%Y-%m-%d-%H-%M-%S')

      csv_name = "rex_client_log_#{current_time}"
      dcsv = Doc.create(responsible: support.id, type: "file", title: "#{csv_name}.csv", tagset_ids: [@tagset_id])
      dcsv.blob = csv
      dcsv.save

      error_length = @errors&.length

      doc_mail = Doc.create(
        responsible: support.id,
        type: 'email', 
        title: "Rapport Syncro REX #{HERMES_CONFIG['environment']} : #{current_time} #{error_length} erreurs", 
      )
      doc_mail.superfields['mail_to'] = (@email_mail_to).join(',')
      doc_mail.superfields['mail_from'] = support.email

      body = "Bonjour, <br/>"
      body += "Voici le lien vers le log de la syncro REX : <a style=\"font-weight: bold;color: #15c !important; \" href=\"https://#{APP_CONFIG["app_host"]}/documents/#{dcsv.id}\">#{dcsv.title}</a><br/>"
      body += "Nombre d'erreurs : #{error_length} <br/>"

      if error_length > 0
        body += "Les #{[100, error_length].min} premières erreurs : <br/>"
        @errors.take(100).each do |error|
          body += "- #{error}<br/>"
        end 
          
        if error_length > 100
          body += "Pour voir les erreurs suivantes, il faut aller sur l'admin PAW.<br/>"
        end 
      else
        body += "Aucune erreur.<br/>"
      end 

      doc_mail.body = body
      doc_mail.save

      # Envoi du mail
      Agent.regular_mail(doc_mail.id.to_s, 
        to: doc_mail.superfields['mail_to'], 
        from: doc_mail.superfields['mail_from'], 
        title: doc_mail.title, 
        content: doc_mail.body
      ).deliver_later
    end 

    def get_token
      @token = RestClient.post(
        @auth_url,
        headers: { "Content-Type" => "application/x-www-form-urlencoded" },
        grant_type: "password",
        client_id: @client_id,
        client_secret: @client_secret,
        username: @username,
        password: @password,
        log: logger,
      ).json_to_h[:access_token]
      puts "Got Rex token : #{@token}"
      @token
    end

    def request_header
      {
        "Content-Type" => "application/x-www-form-urlencoded",
        "Authorization" => "Bearer #{@token}",
      }
    end

    def get_structure_ed(type)
      return @ed_entity if type == "entity"
      return @ed_lessor if type == "lessor"
      return @ed_contact if type == "contact"
      return
    end

    def find_base_structure_by_id(id, type = "entity")
      get_token if !@token

      begin
        return RestClient.get(
          "#{@base_url}/entityDefinition/#{get_structure_ed(type)}/entity/#{id}",
          request_header,
        ).json_to_h
      rescue RestClient::ExceptionWithResponse => e
        return;
      end
    end

    def find_entity_by_id(id)
      find_base_structure_by_id(id, "entity")
    end

    def find_lessor_by_id(id)
      find_base_structure_by_id(id, "lessor")
    end

    def find_structure_by_id(id)
      find_base_structure_by_id(id, "entity") || find_base_structure_by_id(id, "lessor")
    end

    def get_page(opt, type = "entity", page = 1)
      if @logger
        @logger.info "Getting page : #{page}"
      end 
      size = 10
      from = (page - 1) * size
      
      get_token if !@token
      
      query = {
        from: from,
        size: size,
        sortField: "fields.id_cristal",
        sortOrder: 'ASC',
      }.merge(opt).to_query

      start = Time.now

      response = begin
        RestClient.get(
          "#{@base_url}/entityDefinition/#{get_structure_ed(type)}/entity?#{query}",
          request_header,
        )
        rescue RestClient::ExceptionWithResponse => e
          e.response
      end
      

      finish = Time.now
      diff = finish - start

      log_puts "#{get_structure_ed(type)}/entity?#{query} #{diff} sec"

      if response.code == 401
        get_token
        return get_page(opt, type, page) 
      end

      response.body.json_to_h
    end

    def search_base(opt, type = "entity", callback = nil, page = 1, previous_result = [], total = 0)
      start = Time.now
      size = 10
      from = (page - 1) * size
      
      if total && total < from
        return previous_result
      end
      
      get_token if !@token
      
      query = {
        from: from,
        size: size,
        sortField: "fields.id_cristal",
        sortOrder: 'ASC',
      }.merge(opt).to_query

      response = begin
        RestClient.get(
          "#{@base_url}/entityDefinition/#{get_structure_ed(type)}/entity?#{query}",
          request_header,
        )
        rescue RestClient::ExceptionWithResponse => e
          e.response
      end

      if response.code == 401
        get_token
        return search_base(opt, type, callback, page, previous_result, total) 
      end

      r = response.body.json_to_h

      if callback
        r[:items].each do |item|
          callback.call(item)
        end
        
        finish = Time.now
        diff = finish - start

        puts("Sync #{type} #{[from + size, r[:total]].min}/#{r[:total]} #{diff} sec")
      end

      new_result = (callback) ?  [] : previous_result + r[:items]
      
      search_base(opt, type, callback, page + 1, new_result, r[:total])
    end

    def search_entities(opt = {})
      search_base(opt, "entity")
    end

    def search_lessors(opt = {})
      search_base(opt, "lessor")
    end

    def search_contacts(opt = {})
      search_base(opt, "contact")
    end

    def search_structures(opt = {})
      search_base(opt, "entity") + search_base(opt, "lessor")
    end

    def traverse_base(opt, type, callback)
      total = get_page(opt, type, 1)[:total]
      totalPages = (total / 10).floor + 1
      
      (1..totalPages).to_a.each do |page|
        start = Time.now
        reponse = get_page(opt, type, page)
        if callback
          reponse[:items].each do |item|
            callback.call(item)
          end
          
          finish = Time.now
          diff = finish - start

          puts("Sync #{type} #{[page * 10, total].min}/#{total} #{diff} sec")
        end
      end
    end

    def traverse_structures(opt = {}, opt2 = {}, callback = nil)
      traverse_base(opt, 'lessor', callback) unless opt2[:dont_update_lessor] 
      traverse_base(opt, 'entity', callback) unless opt2[:dont_update_entity] 
    end

    def traverse_contacts(opt = {}, opt2 = {}, callback = nil)
      traverse_base(opt, "contact", callback)
    end
  end
end
